@@ -528,7 +528,7 @@ int main(int argc, char **argv)
     };
 
     /**
-     *= https://tools.ietf.org/rfc/rfc7627#section-5.3
+     *= https://www.rfc-editor.org/rfc/rfc7627#section-5.3
      *= type=test
      *# If the original session used the "extended_master_secret"
      *# extension but the new ClientHello does not contain it, the server
@@ -581,7 +581,7 @@ int main(int argc, char **argv)
     };
 
     /**
-     *= https://tools.ietf.org/rfc/rfc7627#section-5.3
+     *= https://www.rfc-editor.org/rfc/rfc7627#section-5.3
      *= type=test
      *# If the original session did not use the "extended_master_secret"
      *# extension but the new ClientHello contains the extension, then the
